#
# Copyright (c) 2002, Jason Erickson <jerickso@stickpeople.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of Jason Erickson nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import string
import time

import vimThread

# The buttonKeeper Class is just a convient way of storing information
#  for a button.  It can store:
#      button string
#      button width
#      button object
#      one other button specific information
#  All are keyed off of the full button path
#
class buttonKeeper:
  def __init__(self):
    # Dictionary that has the file strings associated with the ids
    self.text = {}
    self.button = {}
    self.width = {}
    self.other = {}

    self.rows = []  # Number of rows for the GUI

  def insert(self, path, text, button, width, other = None):
    self.text[path] = button
    self.button[path] = button
    self.width[path] = width
    self.other[path] = other

  def setOther(self, path, other):
    self.other[path] = other

  def getOther(self, path):
    return self.other[path]

  def getKeys(self):
    ret = self.button.keys()
    ret.sort()
    return ret

  def getButton(self, path):
    return self.button[path]

  def getWidth(self, path):
    return self.width[path]

  def delete(self, file):
    del self.text[file]
    del self.button[file]
    del self.width[file]
    del self.other[file]



# The GUI class will load a valid GUI, based upon what is installed on the
#  system.
#
class GUI:
  def __init__(self, queue, files):
    # See what GUI set we have Available to us
    # Order of trying:
    #   wxPython
    #   QT
    #   tkinker
    #
    notfound = 0
    try:
      from wxPython.wx import *
      import GUIwx
      app = GUIwx
    except ImportError:
      try:
        from qt import *
        import GUIqt
        app = GUIqt
      except ImportError:
        try:
          from Tkinter import *
          import GUItk
          app = GUItk
        except ImportError:
          notfound = 1


    # If we found a GUI, start it up
    if notfound == 0:
      a = app.startGUI(queue, files)
    # TODO: Else, lets raise an exception
    else:
      pass



class GUIFramework:
  def __init__(self):
    # Dictionary that has the file strings associated with the ids
    pass

  # A request to remove a button
  def fileRemove(self, file):
    pass

  # A request to add a button
  def fileAdd(self, file):
    pass

  # A notification of what file is currently selected
  def fileSelected(self, file):
    pass
