#
# Copyright (c) 2002, Jason Erickson <jerickso@stickpeople.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of Jason Erickson nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

#
# This file is for OS independence.  Copy this file and make changes for other
#   OSs.  The only OS independence class is the interface to VIM.
#



from win32com.client.dynamic import Dispatch
import pythoncom
import win32event

#
# Class VIMINTERFACE - The Interface to VIM
# All VIM Interfaces must support the following:
#   bringToFront() - Bring the Vim Window to the front
#   command(string) - Send a command to VIM
#   eval(string) - Send an evaluation to VIM
#   threadInitialize() - Called after a thread gets initialized
#
class VIMINTERFACE:
  def __init__(self):
    # Create an interface to VIM
    self.vim = Dispatch("Vim.Application")
    self.vim.SetForeground()

  def threadInitialize(self):
      pythoncom.CoInitializeEx(0)

  def bringToFront(self):
    self.vim.SetForeground()

  def command(self, string):
    self.vim.SendKeys("<C-\><C-N>:" + string + "\n<Esc>")

  def eval(self, string):
    return self.vim.Eval(string)
