#
# Copyright (c) 2002, Jason Erickson <jerickso@stickpeople.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of Jason Erickson nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.



import os
import time


# VIM Interface
#   threadInitialize()
#   select(filename, line#)
#   close(filename)
#   open(filename)
#   fileList()
#

class VIM:
  def __init__(self):
      # Create an interface to VIM
      if os.name == 'nt' or os.name == 'dos':
        import oswin32
        self.vim = oswin32.VIMINTERFACE()
      else:
        raise OSError, "Unknown interface for os %s"%(os.name)


      # Setup the file List dictionary
      self.fileList = {}   # File list is a dictionary keyed on files names
      self.bufferList = {} # Buffer list is a dictionary keyed on buffer nums

      self.vim.bringToFront()

  def threadInitialize(self):
      self.vim.threadInitialize()



  # Choose which file to view, and on what line number
  def select(self, file, lineNumber):
      # Update the fileList variable
      self.files()

      try:
        # Check to make sure that the buffer still exists
        if self.vim.eval("bufexists(" + str(self.fileList[file]) + ")"):
          self.vim.command(":b! " + str(self.fileList[file]))
          self.vim.bringToFront()
        else:
          # TODO: Throw an exception
          pass

      except KeyError:
        # TODO: Throw an exception
        pass

  def selected(self):
    return self.vim.eval('expand("%:p")')


  # Choose which file to close
  def close(self, file):
      # Remove the requested file from VIM and our keepers
      try:
        del self.bufferList[self.fileList[file]]
        del self.fileList[file]
        # If an exception was not thrown, then issue the delete command
        # TODO
      except KeyError:
        pass


  # Choose which file to open
  # TODO: What if the file does not exist (it gets created)
  def open(self, file):
      # Open file the file
      self.vim.command(":e! " + file)

      # Bring VIM to the foreground
      self.vim.bringToFront()

      # TODO: Timeout incase of a failure to load
      while self.vim.eval("bufloaded('" + file + "')") == 0:
        time.sleep(.2)

      self.fileList[file] = self.vim.eval("bufnr('" + file + "')")
      self.bufferList[self.fileList[file]] = file


  # Returns an array of files that are currently in VIM's buffers
  def files(self):
      array = []

      # Request a buffer list and search it
      highestNum = self.vim.eval("bufnr('$')")
      for i in range(1,int(highestNum)+1):
        # Make sure that the buffer exists
        if self.vim.eval('buflisted(' + str(i) + ')') != '0':
          file = self.vim.eval('expand("#' + str(i) + ':p")')


          # Check to see if it is the same as our list
          if self.bufferList.has_key(i) == 1:
            if self.bufferList[i] != file:
              # If there is a new file, modify the lists
              if len(file) != 0:
                self.bufferList[i] = file
                self.fileList[file] = i
              # Else delete the lists
              else:
                del self.fileList[self.bufferList[i]]
                del self.bufferList[i]
          # If it is not in our list, we need to add it
          else:
            if len(file) != 0:
              self.bufferList[i] = file
              self.fileList[file] = i


          # Check the length to determine if a buffer is there
          if len(file) != 0:
            # Convert the \ to a \\ in the name
            array.append(file)

      return array


  def __del__(self):
      pass

