#
# Copyright (c) 2002, Jason Erickson <jerickso@stickpeople.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of Jason Erickson nor the names of its contributors
#   may be used to endorse or promote products derived from this software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.



import time
import thread
import threading
import Queue

import vim


# The class polling thread that 'polls' VIM and checks of any updates.
#   Also checks on the OS queue to see if there is any requests in it

class pollThread:
  def __init__(self, win, queue):
    self.win = win          # The window object
    self.stopThread = 1     # Should the thread be stopped
    self.running = 0        # Is the thread currently running

    # Setup the dictionary to be empty first
    self.dict = {}

    # Initialize a Q
    self.Q = queue

    # Intialize the editor
    self.editor = vim.VIM()

    self.thread = threading.Thread(None, self.run, "pollThread")


  def start(self):
    self.stopThread = 0
    self.running = 1
    self.thread.start()


  def stop(self):
    self.stopThread = 1
    self.thread.join()


  def isRunning(self):
    return self.running


  def run(self):
#    time.sleep(.5)
    self.editor.threadInitialize()
    editor = vim.VIM()
    # While we can still run
    while self.stopThread == 0:
      # Check Q for any additional files
      if self.Q.empty() == 0:
        newfile = self.Q.get()
        editor.open(newfile)

      # Get a list of files from the editor
      list = editor.files()
      list.sort()

      # Make sure that all the keys are updated and that anything added is
      #   there
      for i in list:
        # If the elements are not the same, insert the missing key into the GUI
        if self.dict.has_key(i) != 1:
          self.win.insert(i)
          self.dict[i] = 1

      # Get a list of keys that the thread knows about
      array = self.dict.keys()
      array.sort()

      # Check to see if there is a difference in the number of keys.  If so,
      #   assume that a file was removed from the editor.
      if len(list) != len(array):
        # Use the size of the 'list' since it is the smaller
        for i in range(len(list)):
          while list[i] != array[i]:
            self.win.remove(array[i])
            # Need to remove from both the perminate dictionary and the
            #    temperary sorted dictionary
            del self.dict[array[i]]
            del array[i]

        # If the sizes are still different, it is because the removed files
        #   are at the end of the array
        while len(list) != len(array):
          self.win.remove(array[len(array)-1])
          del self.dict[array[len(array)-1]]
          del array[len(array)-1]

      # Send what the currently selected object is
      self.win.select(editor.selected())
      # Sleep before checking again
      time.sleep(.5)

    self.running = 0


  def select(self, file, lineNum = 0):
    self.editor.select(file, lineNum)


  # TODO: Not implemented yet, message from GUI
  def remove(self, file):
    self.editor.close(file)


  # TODO: Not implemented yet, message from GUI
  def insert(self, file):
    self.editor.open(file)


  def __del__(self):
    self.stop()

